package com.example.demo;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.FactoryBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.AutowiredAnnotationBeanPostProcessor;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.support.DefaultListableBeanFactory;
import org.springframework.beans.factory.support.RootBeanDefinition;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.context.annotation.AnnotationConfigApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.ContextAnnotationAutowireCandidateResolver;
import org.springframework.context.support.GenericApplicationContext;
import org.springframework.core.ResolvableType;
import org.springframework.core.annotation.AnnotationAwareOrderComparator;

import com.fasterxml.jackson.databind.ObjectMapper;

public class MinimalReproductionTests {
	
	@Test
	void aotConfig() {
		try (GenericApplicationContext context = new GenericApplicationContext()) {
			DefaultListableBeanFactory beanFactory = context.getDefaultListableBeanFactory();
			beanFactory.setAutowireCandidateResolver(new ContextAnnotationAutowireCandidateResolver());
			beanFactory.setDependencyComparator(AnnotationAwareOrderComparator.INSTANCE);
			context.registerBeanDefinition("jacksonTesterFactoryBean", getJacksonTesterFactoryBeanBeanDefinition());
			context.refresh();
			prepareConsumer(context);
		}
	}

	@Test
	void javaConfig() {
		try (AnnotationConfigApplicationContext context = new AnnotationConfigApplicationContext(Config.class)) {
			prepareConsumer(context);
		}
	}

	private void prepareConsumer(ConfigurableApplicationContext context) {
		Consumer consumer = new Consumer();
		AutowiredAnnotationBeanPostProcessor beanPostProcessor = new AutowiredAnnotationBeanPostProcessor();
		beanPostProcessor.setBeanFactory(context.getBeanFactory());
		beanPostProcessor.processInjection(consumer);
		System.out.println(consumer.tester);
	}

	@Configuration
	static class Config {

		@Bean
		JacksonTesterFactoryBean jacksonTesterFactoryBean() {
			return new JacksonTesterFactoryBean(new ObjectMapper());
		}

	}

	static class Consumer {

		@Autowired
		JacksonTester<?> tester;

	}

	static class Request {

	}

	static BeanDefinition getJacksonTesterFactoryBeanBeanDefinition() {
		ResolvableType beanType = ResolvableType.forClassWithGenerics(FactoryBean.class,
				ResolvableType.forClassWithGenerics(JacksonTester.class, Object.class));
		RootBeanDefinition beanDefinition = new RootBeanDefinition(beanType);
		beanDefinition.setInstanceSupplier(() -> new Config().jacksonTesterFactoryBean());
		return beanDefinition;
	}

	static class JacksonTester<T> {

		private final ObjectMapper objectMapper;

		public JacksonTester(ObjectMapper objectMapper) {
			this.objectMapper = objectMapper;
		}

	}

	static class JacksonTesterFactoryBean implements FactoryBean<JacksonTester<?>> {

		private final ObjectMapper objectMapper;

		JacksonTesterFactoryBean(ObjectMapper objectMapper) {
			this.objectMapper = objectMapper;
		}

		@Override
		public JacksonTester<?> getObject() throws Exception {
			return new JacksonTester<>(this.objectMapper);
		}

		@Override
		public Class<?> getObjectType() {
			return JacksonTester.class;
		}

	}

}
